# $Id: AlvisRepository.pm,v 1.1 2005/06/17 15:45:56 mike Exp $

package Alvis::OAI::AlvisRepository;
use strict;
use warnings;

use IO::File;

use Alvis::OAI::Repository;
use vars qw(@ISA);
@ISA = qw(Alvis::OAI::Repository);


sub Identify {
    my $this = shift();
    my(%args) = @_;

    return new HTTP::OAI::Identify(
	baseURL => $this->baseURL(),
	adminEmail => 'mike@indexdata.com',
	repositoryName => 'Alvis pipeline repository',
    );
}


sub ListRecords {
    my $this = shift();
    my(%args) = @_;

    if (0) {
	# I need help from the HTTP::OAI author to do this
	return new HTTP::OAI::ListRecords(
	    ### What on earth is supposed to go in here?
	);
    }

    my $date = "2005-06-07T12:08:04Z";###
    my $prefix = "alvis";###
    my $request = "http://localhost:12345/path/to/prog";###
    my $response = <<__EOT__;
<?xml version="1.0" encoding="UTF-8"?>
<OAI-PMH xmlns='http://www.openarchives.org/OAI/2.0/'
         xmlns:xsi='http://www.w3.org/2001/XMLSchema-instance'
         xsi:schemaLocation='http://www.openarchives.org/OAI/2.0/
                             http://www.openarchives.org/OAI/2.0/OAI-PMH.xsd'>
  <responseDate>$date</responseDate>
  <request verb='ListRecords' metadataPrefix='$prefix'>$request</request>
  <ListRecords>
__EOT__

    my @records = $this->find(%args);
    foreach my $record (@records) {
	$response .= $this->format($record);
    }

$response .= <<__EOT__;
  </ListRecords>
</OAI-PMH>
__EOT__
    return $response;
}


sub find {
    my $this = shift();

    my $dir = $this->arg("sourcedir")
	or die "no source directory specified for $this";
    die "no such source directory '$dir'"
	if ! -d $dir;

    my @files = ();
    foreach my $file (glob("$dir/*.xml")) {
	push @files, $file;
    }

    return @files;
}


sub format {
    my $this = shift();
    my($record) = @_;

    my $fh = new IO::File("<$record")
	or die "can't open record '$record': $!";
    my @lines = <$fh>;
    $fh->close();
    unlink($record)	   # This was a temporary copy made by write()
	or die "can't remove de-spooled file '$record': $!";
    shift @lines if $lines[0] =~ /^<\?xml/;
    my $doc = join("", @lines);

    ### The reading half of the shitty OAI module barfs (with an
    #   exceptionally unhelpful error-message) if it runs into a
    #   record with a DTD specification of the <!DOCTYPE> kind.  To
    #   make its life easier, we should probably remove any such thing
    #	just as we did with the <?xml> line, but it's more awkward to
    #	do as such declarations are typically broken across lines, and
    #	may in general include ">" characters.  For now, make do with
    #	the following rather fragile regexp.
    $doc =~ s/(<!DOCTYPE .*?>)//s;

    # Just to make the formatting of the message look nicer.
    $doc =~ s/^/          /gm;

    my $id = "oai:arXiv:cs/temporary-id"; ### should do better
    my $date = "1998-03-18";	### date of file
return <<__EOT__;
    <record>
      <header>
        <identifier>$id</identifier>
        <datestamp>$date</datestamp>
      </header>
      <metadata>
        <oai:metadata xmlns:oai='http://www.openarchives.org/OAI/2.0/'>
$doc
        </oai:metadata>
      </metadata>
    </record>
__EOT__
}


1;

