# $Id: Repository.pm,v 1.1 2005/06/17 15:45:56 mike Exp $

package Alvis::OAI::Repository;
use strict;
use warnings;


# Create a new repository object capable of handling requests.
#
sub new {
    my $class = shift();
    my(%args) = @_;

    return bless {
	baseURL => undef,
	args => \%args,
    }, $class;
}


# Returns the base URL of the repository.  If an argument is provided,
# sets the base URL to its value (returning the old value).  A base
# URL must be set before Identify requests can be handled.
#
sub baseURL {
    my $this = shift();
    my($baseURL) = @_;

    my $old = $this->{baseURL};
    $this->{baseURL} = $baseURL
	if defined $baseURL;
    return $old;
}


# Returns the the value of the specified argument, as specified at
# creation time.
#
sub arg {
    my $this = shift();
    my($key) = @_;

    return $this->{args}->{$key};
}

# Handle an OAI request
# Arguments:
#	$verb - "Identify", "GetRecord" or similar OAI verb
#	%args - hash of arguments appropriate to $verb
# Returns up to three values:
#	$response - if defined, a successful response object
#	$code - if $response undefined, an OAI error-code
#	$message - if $response undefined, a message explaining $code
#
sub handle {
    my $this = shift();
    my($verb, %args) = @_;

    return $this->$verb(%args);
}


sub Identify {
    my $this = shift();
    my(%args) = @_;

    return new HTTP::OAI::Identify(
	baseURL => $this->{baseURL},
	adminEmail => 'mike@indexdata.com',
	repositoryName => 'Dummy repository',
    );
}


1;
