# $Id: Read.pm,v 1.5 2005/06/17 16:56:00 mike Exp $

package Alvis::Pipeline::Read;
use vars qw(@ISA);
@ISA = qw(Alvis::Pipeline);

use strict;
use warnings;

use HTTP::OAI::Harvester;
use HTTP::OAI::Metadata::OAI_DC; # It's stupid that we need this


sub new {
    my $class = shift();
    my(%opts) = @_;

    my $this = bless {}, $class;
    my $baseURL = delete $opts{baseURL};
    my $host = delete $opts{host};
    my $port = delete $opts{port};
    my $path = delete $opts{path};

    die "either baseURL or host (and optionally port and path) must be specified"
	if !defined $baseURL && !defined $host;

    if (!defined $baseURL) {
	$baseURL = "http://$host";
	$baseURL .= ":$port" if defined $port;
	$baseURL .= "/";
	$baseURL .= $path if defined $path;
    }

    $this->{baseURL} = $baseURL;
    $this->{host} = $host;
    $this->{port} = $port;
    $this->{path} = $path;

    $this->_setopts(%opts);

    # The stupid OAI module casually emits warnings (which by the way
    # contain no useful information), so we need to catch and discard
    # the more common and pointless of these, and re-throw the rest.
    local $SIG{__WARN__} = sub {
	my($text) = @_;
	warn $text if $text !~ /Attempt to set a non-existant baseURL/;
    };

    #print "Using baseURL '$baseURL'\n";
    my $harvester = new HTTP::OAI::Harvester(baseURL => $baseURL);
    if (1) {
	# We do an Identify this so that we discover straight away at
	# creation time if what we've connected to is not an OAI
	# server.
	my $response = $harvester->repository($harvester->Identify);
	die("OAI [Identify] error: ",
	    $response->code() . " (" . $response->message(), ")")
	    if $response->is_error();
	### Log the results of the identify operation
    }

    $this->{harvester} = $harvester;
    return $this;
}


sub read {
    my $this = shift();
    my($block) = @_;

    my $list = $this->{list};
    if (!defined $list) {
AGAIN:
	my $prefix = $this->{options}->{metadataPrefix} || 'oai_dc';
	#warn "calling ListRecords";
	$list = $this->{harvester}->ListRecords(metadataPrefix => $prefix,
		handlers => { metadata => 'HTTP::OAI::Metadata' });
	#warn "got ListRecords response $list = ", Dumper($list);
	die("ListRecords: ", $list->code, " (", $list->message, ")")
	    if $list->is_error();
	$this->{list} = $list;
    }

    #warn "calling $list->next()";
    my $rec = $list->next();
    if (!defined $rec) {
	return undef if !$block;
	sleep(10);		### should be configurable.
	goto AGAIN;
    }
    die("next record: ", $rec->code, " (", $rec->message, ")")
	if $rec->is_error();

    #warn "got record $rec";
    my $node = $rec->metadata()->dom()->documentElement();
    #warn "got node $node";

    ### I have no idea why, but for some reason the HTTP::OIA module
    #	seems to wrap the actual data in not one but two <metadata>
    #	elements.  Discard these.
    while ($node->nodeName() eq "metadata") {
	$node = $node->firstChild();
	# skip past an initial text node
	$node = $node->nextSibling() if $node->nodeName() eq "text";
    }

    return $node;
}


sub close {
    my $this = shift();

    # Nothing to do, really.
}


1;
