# $Id: Write.pm,v 1.3 2005/06/17 15:45:56 mike Exp $

package Alvis::Pipeline::Write;
use vars qw(@ISA);
@ISA = qw(Alvis::Pipeline);

use strict;
use warnings;

use IO::File;


sub new {
    my $class = shift();
    my(%opts) = @_;

    my $this = bless {}, $class;
    $this->{port} = delete $opts{port}
	or die "new($class) with no port";
    my $spooldir = $this->{spooldir} = delete $opts{spooldir}
	or die "new($class) with no spooldir";

    if (! -d $spooldir) {
	mkdir($spooldir)
	    or die "can't create spool directory '$spooldir': $!";
	$this->{next} = 0;
    } else {
	### set $this->{next} to point past the end of the spooled files.
    }

    my $loglevel = delete $opts{loglevel};
    $this->_setopts(%opts);

    my $pid = fork();
    die "couldn't fork: $!" if !defined $pid;
    if ($pid == 0) {
	# Child process
	$this->_start_server($loglevel);
    }

    # Automatic reaper prevents the child going zombie when we kill it
    $SIG{CHLD} = 'IGNORE';
    $this->{pid} = $pid;

    return $this;
}


sub write {
    my $this = shift();
    my($xmlDocument) = @_;

    my $spooldir = $this->{spooldir};
    my $filename = sprintf("%s/%08d.xml", $spooldir, $this->{next}++);
    die "reality failure: spool file '$filename' already exists!"
	if -e $filename;

    $xmlDocument = $xmlDocument->toString()
	if ref($xmlDocument);

    my $fh = new IO::File(">$filename")
	or die "can't write spool file '$filename': $!";
    $fh->print($xmlDocument);
    $fh->close();
}


sub close {
    my $this = shift();

    # We need to kill the child process that is running the OAI server
    # so that its Internet port is cleared for subsequent invocations.
    my $pid = $this->{pid};

    kill 15, $pid;
    sleep 1;

    if (kill 0, $pid) {
	warn "kill -15 failed; killing $pid with rude signal 9";
	kill 9, $pid;
	sleep 1;
    }

    if (kill 0, $pid) {
	die "can't kill child server with pid $pid";
    }
}


use Alvis::OAI::AlvisRepository;
use Alvis::OAI::Server;

sub _start_server {
    my $this = shift();
    my($loglevel) = @_;

    my $r = new Alvis::OAI::AlvisRepository(sourcedir => $this->{spooldir});
    my $oai_server = new Alvis::OAI::Server($this->{port}, $r, $loglevel);
    $oai_server->serve();
    die "$oai_server->serve() returned!  It should never do that";
}


1;
