# $Id: 0-creation.t,v 1.3 2005/06/17 15:45:56 mike Exp $

# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl Alvis-Pipeline.t'

#########################

use strict;
use warnings;
use Errno qw(EINVAL);

# change 'tests => 1' to 'tests => last_test_to_print';
use Test::More tests => 11;

BEGIN {
    use_ok('Alvis::Pipeline');
}

#########################

# Insert your test code below, the Test::More module is use()ed here so read
# its man page ( perldoc Test::More ) for help writing this test script.

# Instantiation of abstract subclass
my $pipe;
eval { $pipe = new Alvis::Pipeline() };
ok(!defined $pipe && defined $@ && $@ =~ /Can.t locate object method "new"/,
   "instantiation of abstract pipe subclass is refused");

# Read-pipe creation with missing parameters, and success
eval { $pipe = new Alvis::Pipeline::Read() };
ok(!defined $pipe && defined $@ && $@ =~ /either baseURL/,
   "Read-pipe creation with no host or port is refused");

eval { $pipe = new Alvis::Pipeline::Read(port => "31802") };
ok(!defined $pipe && defined $@ && $@ =~ /either baseURL/,
   "Read-pipe creation with no host is refused");

# This is well-formed, but should fail due to absence of server
eval { $pipe = new Alvis::Pipeline::Read(host => "localhost") };
ok(!defined $pipe && defined $@ && $@ =~ /Not an OAI-PMH response/,
   "Read-pipe creation with no port accepted");

eval { $pipe = new Alvis::Pipeline::Read(host => "localhost",
					 port => 31802) };
ok(!defined $pipe && defined $@ && $@ =~ /Not an OAI-PMH response/,
   "Read-pipe creation with host and port accepted");

# The next two tests rely on access to a well-known OAI archive on the
# Internet.  Each time, we want to detect failure caused by not being
# connected to the Internet, and skip the test in that case.  The
# crappy underlying OAI module doesn't distinguish between this and a
# malformed response, but by inspection $! is EINVAL in the former
# case by 0 in the latter.  D'oh!
#
eval { $pipe = new Alvis::Pipeline::Read(baseURL =>
				"http://www.diva-portal.org/oai/OAI") };
SKIP: {
    skip "not connected to Internet?", 1 if !defined $pipe && $! == EINVAL;
    ok(defined $pipe, "Read-pipe creation with explicit baseURL");
    $pipe->close();
    $pipe = undef;
}

eval { $pipe = new Alvis::Pipeline::Read(host => "www.diva-portal.org",
					 path => "oai/OAI") };
SKIP: {
    skip "not connected to Internet?", 1 if !defined $pipe && $! == EINVAL;
    ok(defined $pipe, "Read-pipe creation with host and path");
    $pipe->close();
    $pipe = undef;
}

# Write-pipe creation with missing parameters, and success
eval { $pipe = new Alvis::Pipeline::Write() };
ok(!defined $pipe && defined $@ && $@ =~ /no port/,
   "Write-pipe creation with no port is refused");

eval { $pipe = new Alvis::Pipeline::Write(port => 29168) };
ok(!defined $pipe && defined $@ && $@ =~ /no spooldir/,
   "Write-pipe creation with no spooldir is refused");

eval { $pipe = new Alvis::Pipeline::Write(port => 29168,
					  spooldir => "/tmp/pipeline") };
ok(defined $pipe,
   "Write-pipe creation with port and spooldir");
$pipe->close();
